@@ -13,7 +13,6 @@ use anyhow::{Context, Result};
 use indexmap::IndexMap;
 
 use sequoia_openpgp as openpgp;
-use sequoia_ipc as ipc;
 use openpgp::{
     Fingerprint,
     KeyHandle,
@@ -79,7 +78,6 @@ pub mod migrate;
 pub mod generate_key;
 pub mod filter;
 pub mod quick;
-pub mod assert_pubkey_algo;
 
 /// Commands and options.
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
@@ -445,7 +443,6 @@ pub enum CmdOrOpt {
     oForceSignKey,
     oForbidGenKey,
     oRequireCompliance,
-    oAssertPubkeyAlgo,
 
     oNoop,
 
@@ -578,6 +575,7 @@ pub struct Config<'store> {
     tofu_default_policy: trust::TofuPolicy,
     trust_model: Option<trust::TrustModel>,
     trusted_keys: Vec<openpgp::Fingerprint>,
+    unwrap_encryption: bool,
     use_tor: OnceCell<bool>,
     verbose: usize,
     verify_options: verify::VerifyOptions,
@@ -592,9 +590,6 @@ pub struct Config<'store> {
     with_tofu_info: bool,
     with_wkd_hash: bool,
 
-    // Backported from GnuPG 2.4.5.
-    pubkey_algo_policy: assert_pubkey_algo::Policy,
-
     // Extension.
     autostart_parcimonie: bool,
 
@@ -662,6 +657,7 @@ impl<'store> Config<'store> {
             force_ownertrust: false,
             groups: Default::default(),
             homedir: std::env::var_os("GNUPGHOME")
+                .filter(|v| ! v.is_empty())
                 .map(Into::into)
                 .ok_or_else(|| anyhow::anyhow!("for conversion to err"))
                 .or_else(|_| homedir::default())?,
@@ -715,6 +711,7 @@ impl<'store> Config<'store> {
             tofu_default_policy: Default::default(),
             trust_model: None,
             trusted_keys: vec![],
+            unwrap_encryption: false,
             use_tor: Default::default(),
             verbose: 0,
             verify_options: Default::default(),
@@ -729,9 +726,6 @@ impl<'store> Config<'store> {
             with_tofu_info: false,
             with_wkd_hash: false,
 
-            // Backported from GnuPG 2.4.5.
-            pubkey_algo_policy: Default::default(),
-
             // Extensions.
             autostart_parcimonie: false,
 
@@ -750,8 +744,8 @@ impl<'store> Config<'store> {
     }
 
     /// Returns an IPC context.
-    pub fn ipc(&self) -> Result<ipc::gnupg::Context> {
-        ipc::gnupg::Context::with_homedir(&self.homedir)
+    pub fn ipc(&self) -> Result<gpg_agent::gnupg::Context> {
+        Ok(gpg_agent::gnupg::Context::with_homedir(&self.homedir)?)
     }
 
     /// Returns a connection to the GnuPG agent.
@@ -904,7 +898,7 @@ impl<'store> Config<'store> {
         P: FnMut(&[u8]) -> Result<()>,
     {
         use gpg_agent::PinentryMode;
-        use gpg_agent::sequoia_ipc::assuan::Response;
+        use gpg_agent::assuan::Response;
 
         let callback = |_agent: &mut _, response| {
             if let Response::Inquire { keyword, parameters } = &response {
@@ -1024,7 +1018,7 @@ impl<'store> Config<'store> {
             // Try to map using groups if `expand_groups` is true.  We
             // don't want to lookup expanded names again, as we may
             // walk into loops.  GnuPG also doesn't do that.
-            Query::Email(e) => if expand_groups {
+            Query::Email(e) | Query::ExactUserID(e) => if expand_groups {
                 if let Some(queries) = self.groups.get(e.as_str()) {
                     let mut acc = Vec::new();
                     for query in queries {
@@ -1184,7 +1178,7 @@ impl<'store> common::Common<'store> for Config<'store> {
         self.outfile.as_ref()
     }
 
-    fn policy(&self) -> &dyn Policy {
+    fn policy(&self) -> &GPGPolicy {
         &self.policy
     }
 
@@ -1571,7 +1565,9 @@ fn print_additional_version(config: &Config) {
         .filter(|a| *a != 2 && *a != 3) // Skip single-use RSA
         .filter(|a| *a != 20) // Skip dual-use ElGamal
         .map(PublicKeyAlgorithm::from)
-        .filter(|a| a.is_supported()).enumerate()
+        .filter(|a| a.is_supported())
+        .filter(|a| config.policy.public_key_algorithm(*a).is_ok())
+        .enumerate()
     {
         if i > 0 {
             w.emit(format_args!(", "));
@@ -1583,7 +1579,9 @@ fn print_additional_version(config: &Config) {
     let mut w = Writer::new("Cipher: ");
     for (i, a) in (0..0xff).into_iter()
         .map(SymmetricAlgorithm::from)
-        .filter(|a| a.is_supported()).enumerate()
+        .filter(|a| a.is_supported())
+        .filter(|a| config.policy.symmetric_algorithm(*a).is_ok())
+        .enumerate()
     {
         if i > 0 {
             w.emit(format_args!(", "));
@@ -1667,11 +1665,6 @@ fn real_main() -> anyhow::Result<()> {
         match arg {
             Argument::Option(aHelp, _) =>
                 return Ok(parser.help(&opt)),
-            Argument::Option(aVersion, _) => {
-                // GnuPG emits a warning on --version.
-                opt.check_homedir_permissions()?;
-                return Ok(parser.version(&opt));
-            },
             Argument::Option(aWarranty, _) =>
                 return Ok(parser.warranty()),
             Argument::Option(aDumpOptions, _) =>
@@ -1721,7 +1714,8 @@ fn real_main() -> anyhow::Result<()> {
                 opt.default_keyring = false;
             },
 
-	    aCheckKeys
+            aVersion
+	        | aCheckKeys
 	        | aListPackets
 	        | aImport
 	        | aFastImport
@@ -1949,12 +1943,12 @@ fn real_main() -> anyhow::Result<()> {
             },
 
 	    oStatusFD => {
-                opt.status_fd =
-                    argparse::utils::sink_from_fd(value.as_int().unwrap())?.into();
+                opt.status_fd.set_stream(
+                    argparse::utils::sink_from_fd(value.as_int().unwrap())?);
             },
 	    oStatusFile => {
-                opt.status_fd =
-                    Box::new(fs::File::create(value.as_str().unwrap())?).into();
+                opt.status_fd.set_stream(
+                    Box::new(fs::File::create(value.as_str().unwrap())?));
             },
 	    oAttributeFD => {
                 opt.attribute_fd = argparse::utils::sink_from_fd(value.as_int().unwrap())?;
@@ -2491,6 +2485,18 @@ fn real_main() -> anyhow::Result<()> {
                 opt.list_options.parse(value.as_str().unwrap())?;
             },
 
+            oDisableCipherAlgo => {
+                let a: babel::Fish<SymmetricAlgorithm> =
+                    value.as_str().unwrap().parse()?;
+                opt.policy.reject_symmetric_algo(a.0);
+            },
+
+            oDisablePubkeyAlgo => {
+                let a: babel::Fish<PublicKeyAlgorithm> =
+                    value.as_str().unwrap().parse()?;
+                opt.policy.reject_public_key_algo(a.0);
+            },
+
             oVerifyOptions => {
                 let options = value.as_str().unwrap();
                 if verify::VerifyOptions::maybe_print_help(options)? {
@@ -2547,6 +2553,8 @@ fn real_main() -> anyhow::Result<()> {
                 opt.policy.weak_digest(
                     value.as_str().unwrap().parse::<babel::Fish<_>>()?.0);
             },
+            oUnwrap => opt.unwrap_encryption = true,
+
             oGroup => {
                 let g = value.as_str().unwrap().splitn(2, "=")
                     .map(|s| s.trim())
@@ -2572,6 +2580,10 @@ fn real_main() -> anyhow::Result<()> {
             },
             oKeyidFormat =>
                 opt.keyid_format = value.as_str().unwrap().parse()?,
+
+            oExitOnStatusWriteError =>
+                opt.status_fd.exit_on_write_error(),
+
             oAutoKeyLocate => {
                 auto_key_locate_given = true;
                 for s in value.as_str().unwrap().split(',') {
@@ -2606,11 +2618,6 @@ fn real_main() -> anyhow::Result<()> {
 
             oForbidGenKey => opt.forbid_gen_key = true,
 
-            // Backported from GnuPG 2.4.5.
-            oAssertPubkeyAlgo =>
-                opt.pubkey_algo_policy.handle_cmdline_arg(
-                    value.as_str().unwrap())?,
-
             // Our own extensions.
             aXSequoiaParcimonie => {
                 set_cmd(&mut command, aXSequoiaParcimonie)?;
@@ -2651,6 +2658,10 @@ fn real_main() -> anyhow::Result<()> {
         }
     }
 
+    if command == Some(aVersion) {
+        return Ok(parser.version(&opt));
+    }
+
     if greeting && ! no_greeting {
         safe_eprintln!("gpg (Sequoia Chameleon {}) {}; \
                    Copyright (C) 2024 Sequoia PGP",
@@ -2895,6 +2906,8 @@ fn real_main() -> anyhow::Result<()> {
         Some(aListPackets) => list_packets::cmd_list_packets(&opt, &args),
         Some(aKeygen) => generate_key::cmd_generate_key(&mut opt, &args, false),
         Some(aFullKeygen) => generate_key::cmd_generate_key(&mut opt, &args, true),
+        Some(aQuickKeygen) =>
+            generate_key::cmd_quick_generate_key(&mut opt, &args),
         Some(aQuickAddKey) => generate_key::cmd_quick_add_key(&mut opt, &args),
         Some(aQuickAddUid) => quick::cmd_quick_add_uid(&mut opt, &args),
         Some(aQuickRevUid) => quick::cmd_quick_revoke_uid(&mut opt, &args),
