@@ -10,11 +10,6 @@ use std::{
 
 use anyhow::{Context, Result};
 
-use sequoia_openpgp as openpgp;
-use openpgp::{
-    policy::Policy,
-};
-
 use sequoia_cert_store as cert_store;
 use cert_store::LazyCert;
 
@@ -27,7 +22,6 @@ mod macros;
 #[macro_use]
 pub mod argparse;
 use argparse::{Argument, Opt, flags::*};
-pub mod assert_pubkey_algo;
 pub mod babel;
 pub mod clock;
 pub mod common;
@@ -58,9 +52,6 @@ pub enum CmdOrOpt {
     oEnableSpecialFilenames,
     oDebug,
 
-    // Backported from GnuPG 2.4.5.
-    oAssertPubkeyAlgo,
-
     // Special, implicit commands.
     aHelp = 'h' as isize,
     aVersion = 32769,
@@ -92,9 +83,6 @@ const OPTIONS: &[Opt<CmdOrOpt>] = &[
     Opt { short_opt: oEnableSpecialFilenames, long_opt: "enable-special-filenames", flags: TYPE_NONE, description: "@", },
     Opt { short_opt: oDebug, long_opt: "debug", flags: TYPE_STRING, description: "@", },
 
-    // Backported from GnuPG 2.4.5.
-    Opt { short_opt: oAssertPubkeyAlgo, long_opt: "assert-pubkey-algo", flags: TYPE_STRING, description: "@", },
-
     // Special, implicit commands.
     Opt { short_opt: aHelp, long_opt: "help", flags: (TYPE_NONE | OPT_COMMAND), description: "@", },
     Opt { short_opt: aVersion, long_opt: "version", flags: (TYPE_NONE | OPT_COMMAND), description: "@", },
@@ -124,9 +112,6 @@ pub struct Config<'store> {
     verbose: usize,
     verify_options: verify::VerifyOptions,
 
-    // Backported from GnuPG 2.4.5.
-    pubkey_algo_policy: assert_pubkey_algo::Policy,
-
     // For sharing the code with with gpg.rs.
     list_only: bool,
 
@@ -148,6 +133,7 @@ impl<'store> Config<'store> {
             debug: 0,
             enable_special_filenames: false,
             homedir: std::env::var_os("GNUPGHOME")
+                .filter(|v| ! v.is_empty())
                 .map(Into::into)
                 .unwrap_or_else(|| dirs::home_dir()
                                 .expect("cannot get user's home directory")
@@ -160,9 +146,6 @@ impl<'store> Config<'store> {
             verbose: 0,
             verify_options: Default::default(),
 
-            // Backported from GnuPG 2.4.5.
-            pubkey_algo_policy: Default::default(),
-
             // For sharing the code with with gpg.rs.
             list_only: false,
 
@@ -215,7 +198,7 @@ impl<'store> common::Common<'store> for Config<'store> {
         self.outfile.as_ref()
     }
 
-    fn policy(&self) -> &dyn Policy {
+    fn policy(&self) -> &GPGPolicy {
         &self.policy
     }
 
@@ -338,11 +321,6 @@ fn real_main() -> anyhow::Result<()> {
                 opt.enable_special_filenames = true;
             },
 
-            // Backported from GnuPG 2.4.5.
-            Argument::Option(oAssertPubkeyAlgo, value) =>
-                opt.pubkey_algo_policy.handle_cmdline_arg(
-                    value.as_str().unwrap())?,
-
             Argument::Option(aHelp, _)
                 | Argument::Option(aVersion, _)
                 | Argument::Option(aWarranty, _)
