@@ -3,7 +3,6 @@ use std::{
     collections::{
         BTreeMap,
         BTreeSet,
-        HashSet,
     },
     io::{self, Write},
     sync::Arc,
@@ -33,7 +32,11 @@ use ipc::Keygrip;
 use sequoia_cert_store as cert_store;
 use cert_store::LazyCert;
 use cert_store::Store;
-use cert_store::store::StoreError;
+use cert_store::store::{StoreError, UserIDQueryParams};
+
+use sequoia_gpg_agent::{
+    keyinfo::KeyInfoList,
+};
 
 use crate::{
     argparse,
@@ -306,6 +309,14 @@ pub fn cmd_list_keys(config: &crate::Config, args: &[String], list_secret: bool)
                     config.keydb().lookup_by_email(&e),
                 Query::UserIDFragment(f) =>
                     config.keydb().grep_userid(&f),
+                Query::ExactUserID(u) =>
+                    config.keydb().select_userid(
+                        UserIDQueryParams::new()
+                            .set_anchor_start(true)
+                            .set_anchor_end(true)
+                            .set_email(false)
+                            .set_ignore_case(false),
+                        &u),
             };
 
             let r = match r {
@@ -427,16 +438,12 @@ where
         config, Some(config.now()), list_all && ! list_secret_keys_mode)?;
     let p = vtm.policy();
 
-    let mut secrets = Default::default();
+    let mut secrets = KeyInfoList::empty();
     if list_secret || (config.with_secret && config.with_colons) {
         if let Ok(mut agent) = config.connect_agent().await {
-            secrets = agent.list_keys().await
-                .map(|keys| {
-                    keys.into_iter()
-                        .map(|k| k.keygrip().clone())
-                        .collect::<HashSet<_>>()
-                })
-                .unwrap_or(Default::default())
+            if let Ok(s) = agent.list_keys().await {
+                secrets = s;
+            }
         }
     }
 
@@ -451,8 +458,7 @@ where
         let mut has_secret: BTreeSet<Fingerprint> = cert
             .keys()
             .filter_map(|k| {
-                let keygrip = Keygrip::of(k.mpis()).ok()?;
-                if secrets.contains(&keygrip) {
+                if secrets.lookup_by_key(&k).is_some() {
                     Some(k.fingerprint())
                 } else {
                     None
@@ -467,7 +473,8 @@ where
             has_secret.insert(skb.fingerprint());
         }
 
-        if list_secret_keys_mode && has_secret.is_empty() {
+        let cert_is_tsk = ! has_secret.is_empty();
+        if list_secret_keys_mode && ! cert_is_tsk {
             // No secret (sub)key, don't list this key in --list-secret-keys.
             continue;
         }
@@ -497,10 +504,19 @@ where
         let have_secret = has_secret.contains(&cert_fp);
         let ownertrust = config.trustdb.get_ownertrust(&cert_fp)
             .unwrap_or_else(|| OwnerTrustLevel::Undefined.into());
+        let token_sn = secrets.lookup_by_key(cert.primary_key().key())
+            .and_then(|i| if let Some(sn) = i.serialno() {
+                Some(TokenSN::SerialNumber(sn.into()))
+            } else if have_secret {
+                Some(TokenSN::SecretAvaliable)
+            } else {
+                None
+            })
+            .or_else(|| cert_is_tsk.then_some(TokenSN::SimpleStub));
 
         Record::Key {
             key: cert.primary_key().key(),
-            have_secret: have_secret && list_secret,
+            have_secret: cert_is_tsk && list_secret,
             validity: acert.cert_validity(),
             expiration_date:  vcert.as_ref()
                 .and_then(|v| v.keys().next().expect("primary key")
@@ -542,7 +558,7 @@ where
                 }
                 kf
             },
-            token_sn: have_secret.then(|| TokenSN::SecretAvaliable),
+            token_sn: token_sn.clone(),
             compliance: cert.primary_key().compliance(config),
         }.emit(config, &mut sink)?;
 
@@ -558,6 +574,13 @@ where
 
         Record::Fingerprint(cert_fp)
             .emit(config, &mut sink)?;
+
+        if ! config.with_colons && config.with_subkey_fingerprint {
+            if let Some(s) = token_sn.and_then(|t| t.pretty_sn()) {
+                writeln!(sink, "      Card serial no. = {}", s)?;
+            }
+        }
+
         if config.with_keygrip
             || (config.with_colons && (list_secret_keys_mode || have_secret))
         {
@@ -673,10 +696,19 @@ where
             let vsubkey = subkey.clone().with_policy(p, config.now()).ok();
             let subkey_fp = subkey.fingerprint();
             let have_secret = has_secret.contains(&subkey_fp);
+            let token_sn = secrets.lookup_by_key(subkey.key())
+                .and_then(|i| if let Some(sn) = i.serialno() {
+                    Some(TokenSN::SerialNumber(sn.into()))
+                } else if have_secret {
+                    Some(TokenSN::SecretAvaliable)
+                } else {
+                    None
+                })
+                .or_else(|| cert_is_tsk.then_some(TokenSN::SimpleStub));
 
             Record::Subkey {
                 key: subkey.key(),
-                have_secret: have_secret && list_secret,
+                have_secret: cert_is_tsk && list_secret,
                 validity: validity,
                 expiration_date:  vsubkey.as_ref()
                     .and_then(|v| v.key_expiration_time()),
@@ -691,7 +723,7 @@ where
                 key_flags: vsubkey.as_ref()
                     .and_then(|v| v.key_flags())
                     .unwrap_or_else(|| KeyFlags::empty()),
-                token_sn: have_secret.then(|| TokenSN::SecretAvaliable),
+                token_sn: token_sn.clone(),
                 compliance: subkey.compliance(config),
             }.emit(config, &mut sink)?;
 
@@ -699,6 +731,13 @@ where
                 Record::Fingerprint(subkey_fp)
                     .emit(config, &mut sink)?;
             }
+
+            if ! config.with_colons && config.with_subkey_fingerprint {
+                if let Some(s) = token_sn.and_then(|t| t.pretty_sn()) {
+                    writeln!(sink, "      Card serial no. = {}", s)?;
+                }
+            }
+
             if config.with_keygrip
                 || (config.with_colons &&
                     (list_secret_keys_mode || have_secret))
