@@ -52,8 +52,12 @@ pub fn cmd_sign(config: &crate::Config, args: &[String],
             }
         })?;
 
+    let last_name = filenames.last().expect("we put a - in if empty");
     let mut sink = if let Some(name) = config.outfile() {
         utils::create(config, name)?
+    } else if detached && last_name != "-" {
+        let ext = if config.armor { "asc" } else { "sig" };
+        utils::create(config, &format!("{}.{}", last_name, ext))?
     } else {
         Box::new(io::stdout())
     };
@@ -169,7 +173,13 @@ pub async fn get_signers(config: &crate::Config<'_>)
         }
 
         let query = local_user.parse::<crate::trust::Query>()?;
-        let certs = config.lookup_certs(&query)?;
+        let mut certs = config.lookup_certs(&query)?;
+
+        // Only keep those with a valid signing (sub)key.
+        certs.retain(|(_, c)| c.to_cert().map(
+            |c| c.keys().with_policy(config.policy(), config.now())
+                .for_signing().next().is_some())
+                     .unwrap_or(false));
 
         // Cowardly refuse any queries that resolve to multiple keys.
         // In my mind, using queries other than fingerprints in
@@ -181,16 +191,21 @@ pub async fn get_signers(config: &crate::Config<'_>)
             0 => return Err(anyhow::anyhow!("Signing key {} not found", query)),
             1 => &certs[0].1,
             n => return Err(anyhow::anyhow!(
-                "Signing key {} maps to {} different keys: {:?}", query, n,
+                "Signing key {} maps to {} different keys: {}", query, n,
                 certs.iter().map(|c| c.1.fingerprint().to_string())
-                    .collect::<Vec<_>>())),
+                    .collect::<Vec<_>>().join(" "))),
         };
 
+        config.policy.public_key_algorithm(
+            cert.primary_key().pk_algo())?;
+
         let vcert = cert.with_policy(config.policy(), config.now())
             .context(format!("Key {} is not valid", query))?;
 
         let mut candidates = Vec::new();
-        for key in vcert.keys().for_signing() {
+        for key in vcert.keys().for_signing()
+            .filter(|k| config.policy.public_key_algorithm(k.pk_algo()).is_ok())
+        {
             if let Ok(signer) = config.get_signer(&vcert, &key).await {
                 candidates.push((key.alive().is_ok(),
                                  key.creation_time(),
