@@ -28,6 +28,12 @@ fn password_store_git() -> Result<()> {
     Experiment::new("password-store-git")?.with_null_policy()?.run()
 }
 
+#[test]
+#[ntest::timeout(600000)]
+fn debsig_verify_0_30() -> Result<()> {
+    Experiment::new("debsig-verify-0.30")?.run()
+}
+
 // The framework:
 
 const EDIT_DISTANCE_CUTOFF: usize = 1000;
@@ -269,6 +275,15 @@ impl<'env> Context<'env> {
             }
         }
 
+        // Make keyring paths absolue.  If they are relative, they are
+        // interpreted as relative to GNUPGHOME.
+        for i in 0..args.len() - 1 {
+            if args[i] == "--keyring" {
+                args[i + 1] = self.recorder_dir.join(&args[i + 1])
+                    .display().to_string();
+            }
+        }
+
         c.args(executable[1..].iter());
         c.arg("--homedir").arg(self.gnupghome());
 
@@ -779,7 +794,7 @@ impl Experiment {
                 move |(what, _, _, distance)|
                 (i, what, distance.unwrap_or(EDIT_DISTANCE_CUTOFF))))
         {
-            let mut summary = streams.entry(what)
+            let summary = streams.entry(what)
                 .or_insert_with(|| StreamSummary {
                     histogram: Histogram::with_buckets(20),
                     worst_index: 0,
